export type Package = {
    name: string;
    version: string;
    description: string;
    author: string;
    license: string;
    private?: boolean;
};
export type BaseArgument = {
    optional?: boolean;
    default?: any;
    short?: string;
    description?: string;
};
export type BooleanArgument = BaseArgument & {
    type: BooleanConstructor;
    default?: boolean;
};
export type NumberArgument = BaseArgument & {
    type: NumberConstructor;
    default?: number;
};
export type StringArgument = BaseArgument & {
    type: StringConstructor;
    default?: string;
};
export type DateArgument = BaseArgument & {
    type: DateConstructor;
    default?: never;
};
export type ObjectType = BaseArgument & {
    type: Record<string | number | symbol, any>;
    default?: boolean;
};
export type Argument = BooleanArgument | NumberArgument | StringArgument | DateArgument | ObjectType;
export type Arguments = Record<string, Argument>;
export type ArgumentType<TArgument extends Argument> = TArgument['type'] extends BooleanConstructor ? boolean : TArgument['type'] extends StringConstructor ? string : TArgument['type'] extends NumberConstructor ? number : TArgument['type'] extends DateConstructor ? Date : TArgument['type'] extends Record<string | number | symbol, infer T> ? T : never;
export type ExtractArgument<TArgument extends Argument> = TArgument['optional'] extends true ? ArgumentType<TArgument> | null : ArgumentType<TArgument>;
export type ExtractArguments<TArguments extends Arguments> = {
    [TName in keyof TArguments]: ExtractArgument<TArguments[TName]>;
};
export type Args<TArguments extends Arguments> = {
    [TName in keyof TArguments]: TArguments[TName];
};
export type Program<TArguments extends Arguments> = {
    argv?: Array<string>;
    args: Args<TArguments>;
};
